import os
import sys


TEST_DIR = os.path.dirname(__file__)
SAMPLE_DATA_DIR = os.path.join(TEST_DIR, 'sample_data')

MADEYE_IMAGE_URL = (
    'https://vignette.wikia.nocookie.net/p__/images/7/73/Mad-Eye_Moody%27s_%22mad_eye%22.jpeg/revision/latest?path-prefix=protagonist'  # noqa: E501
)
STABLE_LOCAL_MADEYE_IMAGE_FILENAME = 'madeye.png'
STABLE_LOCAL_MADEYE_IMAGE_PATH = os.path.join(SAMPLE_DATA_DIR, STABLE_LOCAL_MADEYE_IMAGE_FILENAME)
TEMP_LOCAL_MADEYE_IMAGE_FILENAME_DEFAULT = 'latest?path-prefix=protagonist.png'
TEMP_LOCAL_MADEYE_IMAGE_PATH_DEFAULT = os.path.join(SAMPLE_DATA_DIR,
                                                    TEMP_LOCAL_MADEYE_IMAGE_FILENAME_DEFAULT
                                                    )
TEMP_LOCAL_MADEYE_IMAGE_FILENAME_CUSTOM = 'revision.png'
TEMP_LOCAL_MADEYE_IMAGE_PATH_CUSTOM = os.path.join(SAMPLE_DATA_DIR,
                                                   TEMP_LOCAL_MADEYE_IMAGE_FILENAME_CUSTOM
                                                   )


PHILOSOPHERS_STONE_IMAGE_URL = (
    'https://vignette.wikia.nocookie.net/smurfsfanon/images/3/32/Philosopher%27s_Stone.jpg'
)
STABLE_LOCAL_PHILOSOPHERS_STONE_IMAGE_FILENAME = 'philosophers_stone.png'
STABLE_LOCAL_PHILOSOPHERS_STONE_IMAGE_PATH = os.path.join(
    SAMPLE_DATA_DIR, STABLE_LOCAL_PHILOSOPHERS_STONE_IMAGE_FILENAME
)
TEMP_LOCAL_PHILOSOPHERS_STONE_IMAGE_FILENAME_DEFAULT = 'Philosopher%27s_Stone.png'
TEMP_LOCAL_PHILOSOPHERS_STONE_IMAGE_PATH_DEFAULT = os.path.join(
    SAMPLE_DATA_DIR, TEMP_LOCAL_PHILOSOPHERS_STONE_IMAGE_FILENAME_DEFAULT
)
TEMP_LOCAL_PHILOSOPHERS_STONE_IMAGE_FILENAME_CUSTOM = '32.png'
TEMP_LOCAL_PHILOSOPHERS_STONE_IMAGE_PATH_CUSTOM = os.path.join(
    SAMPLE_DATA_DIR, TEMP_LOCAL_PHILOSOPHERS_STONE_IMAGE_FILENAME_CUSTOM
)


import logging
import os
from typing import List, Tuple

import cv2 as cv
from joblib import delayed, Parallel
import numpy as np
from tqdm import tqdm


def resize_multiple_files(paths: List[str],
                          shape: Tuple[int, int],
                          outdir: str,
                          n_jobs: int
                          ) -> None:
    """
    Resize image files and write them to specified output directory

    Parameters
    ----------
    inpaths
        Paths to image files
    shape
        Desired output shape in the form (height, width)
    outdir
        Desired output path
    n_jobs
        Number of jobs to run in parallel
    """
    if not os.path.isdir(outdir):
        os.makedirs(outdir)
    Parallel(n_jobs=n_jobs)(
        delayed(_resize_file_catch_cv_errors)(
            inpath=path,
            shape=shape,
            outpath=os.path.join(outdir, os.path.basename(path))
        ) for path in tqdm(paths)
    )


def _resize_file_catch_cv_errors(inpath: str, shape: Tuple[int, int], outpath: str) -> None:
    try:
        resize_file(inpath, shape, outpath)
    except cv.error as e:
        logging.error(f'Error on {inpath}', e)


def resize_file(inpath: str, shape: Tuple[int, int], outpath: str) -> None:
    """
    Resize a single image file and write it to the specified output path

    Parameters
    ----------
    inpath
        Path to image file
    shape
        Desired output shape in the form (height, width)
    outpath
        Desired output path
    """
    image_in = cv.imread(inpath)
    image_out = resize_image(image_in, shape)
    cv.imwrite(outpath, image_out)


def resize_image(image: np.array, shape: Tuple[int, int]) -> np.array:
    """
    Resize input image to the specified shape.

    Parameters
    ----------
    image:
        NumPy array with two spatial dimensions and optionally an
        additional channel dimension
    shape:
        Desired output shape in the form (height, width)

    Returns
    -------
    NumPy array with specified shape
    """
    resized = cv.resize(image, dsize=shape[::-1])
    return resized



import os

import cv2 as cv
import pytest

from creevey import resize_image, resize_multiple_files
from .conftest import (SAMPLE_DATA_DIR,
                       STABLE_LOCAL_MADEYE_IMAGE_FILENAME,
                       STABLE_LOCAL_MADEYE_IMAGE_PATH,
                       STABLE_LOCAL_PHILOSOPHERS_STONE_IMAGE_FILENAME,
                       STABLE_LOCAL_PHILOSOPHERS_STONE_IMAGE_PATH
                       )


@pytest.fixture
def sample_image():
    # Image is 396 x 255 x 3
    return cv.imread(STABLE_LOCAL_MADEYE_IMAGE_PATH)


def test_upsize(sample_image):
    expected_spatial_shape = (400, 500)
    upsized_image = resize_image(sample_image, shape=expected_spatial_shape)
    expected_shape = expected_spatial_shape + (3,)
    assert upsized_image.shape == expected_shape


def test_downsize(sample_image):
    expected_spatial_shape = (300, 100)
    upsized_image = resize_image(sample_image, shape=expected_spatial_shape)
    expected_shape = expected_spatial_shape + (3,)
    assert upsized_image.shape == expected_shape


@pytest.fixture
def sample_image_gray():
    image = cv.imread(STABLE_LOCAL_MADEYE_IMAGE_PATH)
    image_gray = cv.cvtColor(image, cv.COLOR_BGR2GRAY)
    return image_gray


def test_grayscale(sample_image_gray):
    expected_shape = (400, 500)
    resized_image = resize_image(sample_image_gray, shape=expected_shape)
    assert resized_image.shape == expected_shape


@pytest.fixture
def resized_files():
    outdir = os.path.join(SAMPLE_DATA_DIR, 'tmp')
    if not os.path.isdir(outdir):
        os.makedirs(outdir)
    paths = STABLE_LOCAL_MADEYE_IMAGE_PATH, STABLE_LOCAL_PHILOSOPHERS_STONE_IMAGE_PATH
    resize_multiple_files(paths=paths,
                          shape=(224, 224),
                          outdir=outdir,
                          n_jobs=20,
                          )
    outpaths = [os.path.join(outdir, STABLE_LOCAL_MADEYE_IMAGE_FILENAME),
                os.path.join(outdir, STABLE_LOCAL_PHILOSOPHERS_STONE_IMAGE_FILENAME)
                ]

    yield outpaths

    for path in outpaths:
        print(path)
        os.remove(path)
    os.rmdir(outdir)


def test_resize_multiple_files(resized_files):
    for path in resized_files:
        image = cv.imread(path)
        assert image.shape == (224, 224, 3)
